/* $Header: /fridge/cvs/cgi.targeted/targeted-config.c,v 1.13 2002/06/30 19:28:40 justins Exp $ */
/*
   
   targeted-config.c          Copyright(c) 2002-1998 Justin David Smith
   justins(at)chaos2.org      http://chaos2.org/
    
   Script configuration and environment
    

   This program is free software; you can redistribute it and/or modify 
   it under the terms of the GNU General Public License as published by 
   the Free Software Foundation, version 2 of the License ONLY.

   This program is distributed in the hope that it will be useful, 
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation, 
   Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/
#include <assert.h>
#include <libjtree.h>
#include <targeted-config.h>


static inline void _new_ip_number(config *c, const char *ipmask) {

   ip_number *m;
   
   /* Attempt to parse the mask */
   m = ip_number_new(ipmask);
   if(m == NULL) return;
   
   /* Add the mask to the admininistrative IP list */
   slink_prepend(&c->admin_ip, m, (destructorfn)ip_number_free);

}


void free_config(config *c) {
/* Releases all data associated with the config structure <c>, including
   any strings in the configuration */

   if(!c) return;
   cgi_conf_free_entries(&c->cgi);
   cgi_env_free_entries(&c->env);
   strreleasea(&c->dir_prefix_fs);
   strreleasea(&c->dir_prefix_uri);
   strreleasea(&c->cache_uri);
   strreleasea(&c->cache_fs);
   strreleasea(&c->cgi_uri);
   strreleasea(&c->header_name);
   strreleasea(&c->htaccess_name);
   strreleasea(&c->logfile);
   strreleasea(&c->css_uri);
   strreleasea(&c->icon_uri);
   slink_release(&c->admin_ip);
   slink_release(&c->index_names);
   free(c);

}


static inline void _validate_filename(stringa *path) {
/* Make sure the pathname is well-formed: begins with /, contains no
   repeated //'s, and does NOT end with a / */

   while(strreplacea(path, "//", "/") > 0) /* Repeat */;
   
   /* Make sure we begin with '/' */
   if(stra(path)[0] != '/') strinserta(path, 0, "/");
   
   /* Trim off any trailing '/' */
   if(stra(path)[strlena(path) - 1] == '/') stra(path)[strlena(path) - 1] = '\0';
   
}


static inline void _validate_pathname(stringa *path) {
/* Make sure the pathname is well-formed: begins with /, contains no
   repeated //'s, and ends with /. */

   /* Make sure we're a valid filename */
   _validate_filename(path);

   /* We just trimmed trailing slash, put it back */
   strconcata(path, "/");

}


config *get_config(const char *filename) {
/* Reads the specified <filename> to get information on the configuration
   of statDB for this session. */

   stringa *buf   =  strnewa(NULL);
   stringa *var   =  strnewa(NULL);
   stringa *val   =  strnewa(NULL);
   config *c      =  NULL;
   int ln         =  0;
   FILE *f;
   if(!filename) filename = "/etc/statdb.conf";

   if(!(c = (config *)malloc(sizeof(config)))) {
      printf("Cannot allocate memory for config.\n");
      return(0);
   }
   c->dir_prefix_fs     = strnewa("/targeted/");
   c->dir_prefix_uri    = strnewa("/");
   c->cache_fs          = strnewa("/httpd/cgi/targeted/");
   c->cache_uri         = strnewa("/cgi/targeted/");
   c->logfile           = strnewa("/httpd/cgi/targeted.log");
   c->cgi_uri           = strnewa("/cgi/targeted.cgi");
   c->header_name       = strnewa(".HEADER");
   c->htaccess_name     = strnewa(".htaccess");
   c->css_uri           = strnewa("/cgi/targeted.css");
   c->icon_uri          = strnewa("/cgi/targeted.jpg");
   c->index_names       = NULL;
   c->admin_ip          = NULL;
   c->thumbnail_width   = 150;
   c->thumbnail_cols    = 4;
   c->use_noncgi_url    = 1;
   c->use_toolbox       = 0;
   cgi_conf_init(&c->cgi);
   cgi_env_init(&c->env);

   if(!(f = fopen(filename, "r"))) {
      printf("Cannot read config file '%s'.\n", filename);
      return(0);
   }
   
   while(freada(buf, f)) {
      ln++;
      trimcommenta(buf, ';');
      if(getassignaa(var, val, buf)) {
         if (cgi_conf_option(&c->cgi, var, val)) 
            /* Already processed */;

         /* Filenames */
         else if(strequala(var, "targetedcgi") || strequala(var, "targeted_cgi_uri"))
            strcopyaa(c->cgi_uri, val);
         else if(strequala(var, "css_url") || strequala(var, "css_uri"))
            strcopyaa(c->css_uri, val);
         else if(strequala(var, "icon_url") || strequala(var, "icon_uri"))
            strcopyaa(c->icon_uri, val);
         else if(strequala(var, "cache") || strequala(var, "cache_dir_uri"))
            strcopyaa(c->cache_uri, val);
         else if(strequala(var, "indexdir") || strequala(var, "cache_dir_fs")) 
            strcopyaa(c->cache_fs, val);
         else if(strequala(var, "directory") || strequala(var, "directory_fs")) 
            strcopyaa(c->dir_prefix_fs, val);
         else if(strequala(var, "directory_uri")) 
            strcopyaa(c->dir_prefix_uri, val);
         else if(strequala(var, "logfile") || strequala(var, "logfile_fs"))
            strcopyaa(c->logfile, val);
         else if(strequala(var, "headerfile") || strequala(var, "header_name")) 
            strcopyaa(c->header_name, val);
         else if(strequala(var, "htaccess_name"))
            strcopyaa(c->htaccess_name, val);
         else if(strequala(var, "index_name"))
            slink_prepend(&c->index_names, strnewa(stra(val)), (destructorfn)strfreea);

         /* General configuration */
         else if(strequala(var, "use_toolbox"))
            c->use_toolbox = strtointa(val, 0);
         else if(strequala(var, "thumbnail_width"))
            c->thumbnail_width = strtointa(val, 0);
         else if(strequala(var, "thumbnail_cols"))
            c->thumbnail_cols = strtointa(val, 0);
         else if(strequala(var, "use_noncgi_url"))
            c->use_noncgi_url = strtointa(val, 0);

         /* Security configuration */
         else if(strequala(var, "administrative_ip")) 
            _new_ip_number(c, stra(val));

         /* Unknown directive */
         else printf("<b>targeted error:</b> Invalid equation in config, line %d\n", ln);
      }
   }

   /* Make sure all pathnames are well-formed */
   _validate_pathname(c->cache_fs);
   _validate_pathname(c->cache_uri);
   /* Next two are treated as files since they are ALWAYS a prefix */
   _validate_filename(c->dir_prefix_fs);
   _validate_filename(c->dir_prefix_uri);
   _validate_filename(c->logfile);
   _validate_filename(c->cgi_uri);
   _validate_filename(c->css_uri);
   _validate_filename(c->icon_uri);
   
   fclose(f);
   strreleasea(&buf);
   strreleasea(&val);
   strreleasea(&var);
   
   return(c);

}
