/* $Header: /fridge/cvs/cgi.targeted/targeted-tree.c,v 1.14 2002/07/03 02:19:24 justins Exp $ */
/*
   
   targeted-tree.c            Copyright(c) 2002-1998 Justin David Smith
   justins(at)chaos2.org      http://chaos2.org/
    
   Functions for displaying directory trees
    

   This program is free software; you can redistribute it and/or modify 
   it under the terms of the GNU General Public License as published by 
   the Free Software Foundation, version 2 of the License ONLY.

   This program is distributed in the hope that it will be useful, 
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation, 
   Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/


#include <targeted.h>
#include <targeted-util.h>
#include <targeted-view.h>
#include <targeted-tree.h>
#include <targeted-list.h>
#include <targeted-log.h>
#include <targeted-cgi.h>
#include <libjtree.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <dirent.h>
#include <unistd.h>


typedef struct _print_data {
   const config *c;
   const char *r_path;
   int depth;
   int indent;
} print_data;


#define  MAX_DEPTH      5


void targeted_tree_entry(const config *c, int depth, int indent, const char *r_path);


static bool _print_dir(const void *_data, void *_params) {

   char path[0x1000];
   char r_path[0x1000];
   const char *d_name = (const char *)_data;
   const print_data *params = (const print_data *)_params;
   struct stat s;
   bintree *cgi;

   cgi = clone_cgi_arguments(params->c->env.query_args);
   if(!strequal(d_name, "..")) {
      qualify_rel_path_file(params->c, r_path, sizeof(r_path), params->r_path, d_name);
      strconcatb(r_path, "/", sizeof(r_path));
      qualify_full_path(params->c, path, sizeof(path), r_path);
      if(targeted_check_access(params->c, r_path) == ACCESS_OK) {
         if(stat(path, &s) == 0) {
            printf("<li><a title=\"Descend into directory: %s\" ", d_name);
            if(targeted_index_present(params->c, r_path)) {
               printf("href=\"%s%s\">", stra(params->c->dir_prefix_uri), r_path);
            } else {
               set_cgi_argument(cgi, "mode", "thumb");
               set_cgi_argument(cgi, "path", r_path);
               construct_cgi_url(params->c, cgi, NORMAL_URL, path, sizeof(path));
               printf("href=\"%s\">", path);
            }
            printf("%s</a>", d_name);
            targeted_tree_entry(params->c, params->depth - 1, params->indent + 2, r_path);
            printf("</li>\n");
         }
      }
   }
   bintree_release(&cgi);
   return(true);

}


void targeted_tree_entry(const config *c, int depth, int indent, const char *r_path) {

   bintree *treedirs;
   print_data params;

   if(depth <= 0) return;
   assert(c != NULL);
   assert(r_path != NULL);
   
   /* Check for existence of the path; open it */
   treedirs = targeted_list_dir(c, r_path, is_directory);
   if(treedirs == NULL) {
      targeted_log_error(c, "unable to open directory in tree view");
      printf("<p>Invalid directory specified, or you do not have access rights to this directory.</p>\n");
   } else if(bintree_count(treedirs) > 1 /* We'll ignore .. */) {
      /* Print subtree header */
      if(indent == 0) {
         printf("<ul style=\"margin: 0em; padding: 0em; list-style-type: none;\">\n");
      } else {
         printf("<ul style=\"margin: 0em; padding: 0em; list-style-type: none; padding-left: 1.5em;\">\n");
      }
      
      /* Walk through the trees in alpha order */
      params.c = c;
      params.r_path = r_path;
      params.depth = depth;
      params.indent = indent;
      bintree_walk(treedirs, _print_dir, &params);
      
      /* Release the trees */
      bintree_release(&treedirs);
      
      /* Print subtree footer */
      printf("</ul>\n");
   }

}


static void _targeted_tree(const config *c, const char *in_r_path, int depth) {

   stringa *r_path;
   
   /* Make sure depth is legal */
   if(depth <= 0) depth = 1;
   if(depth > MAX_DEPTH) depth = MAX_DEPTH;

   /* Rewrite the directory name */
   r_path = rewrite_directory_name(in_r_path);
   
   /* Print the tree */
   targeted_tree_entry(c, depth, 0, stra(r_path));
   
   strreleasea(&r_path);
   
}


void targeted_tree_qstr(const config *c, const char *in_r_path) {

   stringa *arg_depth = strnewa(NULL);
   stringa *arg_r_path = strnewa(NULL);

   assert(c != NULL);
   assert(in_r_path != NULL);
   
   /* Extract the path and arg */
   if(getnumargs(in_r_path, ':') != 2) {
      targeted_log_error(c, "bogus argument format in tree view");
      printf("<p>Invalid argument format.</p>");
      return;
   }
   getarga(arg_depth, in_r_path, 0, ':');
   getarga(arg_r_path, in_r_path, 1, ':');
   
   _targeted_tree(c, stra(arg_r_path), atoi(stra(arg_depth)));
   
   strreleasea(&arg_depth);
   strreleasea(&arg_r_path);

}


void targeted_tree(const config *c) {

   const char *path;
   int depth;

   assert(c != NULL);
   
   path = lookup_cgi_argument(c->env.query_args, "path", "");
   depth = atoi(lookup_cgi_argument(c->env.query_args, "depth", "1"));
   _targeted_tree(c, path, depth);

}
