/* $Header: /fridge/cvs/cgi.targeted/targeted-view.c,v 1.15 2002/07/03 02:19:24 justins Exp $ */
/*
   
   targeted-view.c            Copyright(c) 2002-1998 Justin David Smith
   justins(at)chaos2.org      http://chaos2.org/
    
   Functions for doing view mode
    

   This program is free software; you can redistribute it and/or modify 
   it under the terms of the GNU General Public License as published by 
   the Free Software Foundation, version 2 of the License ONLY.

   This program is distributed in the hope that it will be useful, 
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation, 
   Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/


#include <targeted.h>
#include <targeted-util.h>
#include <targeted-view.h>
#include <targeted-list.h>
#include <targeted-nav.h>
#include <targeted-log.h>
#include <targeted-cgi.h>
#include <libjtree.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <dirent.h>
#include <unistd.h>


typedef struct _print_data {
   const config *c;
   const char *r_path;
   const char *action;
} print_data;


static bool _print_file(const void *_data, void *_params) {
/* Print an individual file from a tree */

   const char *d_name = (const char *)_data;
   const print_data *params = (const print_data *)_params;
   char filename[0x1000];
   char buf[0x1000];
   struct tm *time;
   struct stat s;
   bintree *cgi;
   
   /* Stat the directory entry */
   cgi = clone_cgi_arguments(params->c->env.query_args);
   qualify_full_path_file(params->c, filename, sizeof(filename), params->r_path, d_name);
   if(is_ascii(filename)) {
      if(stat(filename, &s) == 0) {
         /* We can list it! */
         if(strequal(d_name, "..")) {
            /* Backpath */
            printf("<tr class=listdir>");
            if(compute_parent_directory(buf, sizeof(buf), params->r_path)) {
               printf("<td class=left>");
               if(targeted_index_present(params->c, buf)) {
                  printf("<a title=\"Return to parent directory\" href=\"%s%s\">Parent directory ...</a></td>", stra(params->c->dir_prefix_uri), buf);
               } else {
                  set_cgi_argument(cgi, "mode", params->action);
                  set_cgi_argument(cgi, "path", buf);
                  construct_cgi_url(params->c, cgi, NORMAL_URL, buf, sizeof(buf));
                  printf("<a title=\"Return to parent directory\" href=\"%s\">Parent directory ...</a></td>", buf);
               }
            } else {
               printf("<td class=left>%s</td>", d_name);
            }

         } else if(S_ISDIR(s.st_mode)) {
            /* Normal subdirectory */
            printf("<tr class=listdir>");
            qualify_rel_path_file(params->c, buf, sizeof(buf), params->r_path, d_name);
            strconcatb(buf, "/", sizeof(buf));
            printf("<td class=left>");
            printf("<a title=\"Descend to directory: %s\" ", d_name);
            if(targeted_index_present(params->c, buf)) {
               printf("href=\"%s%s%s/\">", stra(params->c->dir_prefix_uri), params->r_path, d_name);
            } else {
               sbprintf(buf, sizeof(buf), "%s%s", params->r_path, d_name);
               set_cgi_argument(cgi, "mode", params->action);
               set_cgi_argument(cgi, "path", buf);
               construct_cgi_url(params->c, cgi, NORMAL_URL, buf, sizeof(buf));
               printf("href=\"%s\">", buf);
            }
            printf("%s/</a></td>", d_name);

         } else if(S_ISLNK(s.st_mode)) {
            /* Normal softlink */
            printf("<tr class=listfile>");
            printf("<td class=left>%s@</a></td>", d_name);

         } else if(S_ISREG(s.st_mode)) {
            /* Normal file */
            printf("<tr class=listfile>");
            printf("<td class=left><a title=\"Display file: %s\" href=\"%s%s%s\">%s</a></td>", d_name, stra(params->c->dir_prefix_uri), params->r_path, d_name, d_name);

         } else {
            /* Unknown; omit */
            printf("<tr class=error>");
            printf("<td class=left>%s?</a></td>", d_name);
         }
         printf("\n");
         
         /* Emit remaining parametres */
         if(S_ISDIR(s.st_mode)) {
            printf("   <td>(Directory)</td>\n");
         } else {
            printf("   <td>%d</td>\n", (int)s.st_size);
         }
         
         /* Figure out the date/time */
         time = localtime(&s.st_mtime);
         printf("   <td class=right>%04d.%02d.%02d %02d:%02d</td>", time->tm_year + 1900, time->tm_mon + 1, time->tm_mday, time->tm_hour, time->tm_min);
         printf("</tr>\n");
      } /* Stat succeeded */
   } /* Filename contains legal characters */

   bintree_release(&cgi);   
   return(true);

}


bool targeted_view_okay(const config *c, const char *r_path, bool isdir) {
/* Return true if viewing a directory list here is okay. */

   int acc;

   assert(c != NULL);
   assert(r_path != NULL);

   if(isdir && !validate_directory_name(r_path)) {
      targeted_log_error(c, "invalid directory specification");
      printf("<p class=error><b>targeted error:</b> Invalid path specified.</p>\n");
      return(false);
   }
   if(!isdir && !validate_file_name(r_path)) {
      targeted_log_error(c, "invalid filename specification");
      printf("<p class=error><b>targeted error:</b> Invalid file specified.</p>\n");
      return(false);
   }
   acc = targeted_check_access(c, r_path);
   if(acc != ACCESS_OK) {
      targeted_log_error_access(c, acc);
      printf("<p class=error><b>targeted error:</b> This file does not exist, or you do not have access to this directory/file.</p>\n");
      return(false);
   }
   if(targeted_index_present(c, r_path)) {
      targeted_log_error(c, "no access allowed when index.(s)html present");
      printf("<p class=error><b>targeted error:</b> Not allowed to give a directory listing when an index file is present. Try <a title=\"Display index file for this directory\" href=\"%s%s\">this URL</a> instead.</p>\n", stra(c->dir_prefix_uri), r_path);
      return(false);
   }
   return(true);

}


static int _count_tree(const bintree *tree) {
/* Count the number of elements in a tree */

   return(bintree_walk(tree, bintree_walk_count, NULL));
   
}


void targeted_view_directory(const config *c, const char *action, const char *r_path) {
/* Views a directory.  action specifies what action to take when clicking on 
   another directory here;  action may be any targeted.cgi action which will
   accept a relative pathname. */

   bintree *dirlist;
   bintree *filelist;
   print_data params;

   assert(c != NULL);
   assert(action != NULL);
   assert(r_path != NULL);
   
   /* Validate the path and construct a real pathname */
   if(!targeted_view_okay(c, r_path, true)) {
      targeted_warning();
      return;
   }
   
   /* Try to load the trees */
   dirlist = targeted_list_dir(c, r_path, is_directory);
   filelist = targeted_list_dir(c, r_path, not_is_directory);
   if(dirlist == NULL || filelist == NULL) {
      targeted_log_error(c, "unable to open specified directory");
      printf("<p class=error><b>targeted error:</b> Invalid directory specified, or you do not have access rights to this directory.</p>\n");
      targeted_warning();
      return;
   }

   /* Process files in this directory */
   printf("<h2>Directory Listing</h2>\n");
   printf("<table class=listing summary=\"Directory listing\">");
   printf("<tr><th class=left>Filename</th><th>Size (bytes)</th><th class=right>Last modified</th></tr>\n");

   /* Walk through the trees in alpha order */
   params.c = c;
   params.r_path = r_path;
   params.action = action;
   bintree_walk(dirlist, _print_file, &params);
   bintree_walk(filelist, _print_file, &params);
   printf("</table>\n\n");
   
   printf("<p>Total:  %d directories / %d files</p>\n", _count_tree(dirlist), _count_tree(filelist));
   
   /* Release the trees */
   bintree_release(&dirlist);
   bintree_release(&filelist);

}


static void _targeted_view(const config *c, const char *in_r_path) {
/* Views a directory listing */

   stringa *r_path;

   /* We are go */
   r_path = rewrite_directory_name(in_r_path);
   targeted_begin(c, stra(r_path));
   targeted_header(c, stra(r_path), false);

   /* View the directory */
   targeted_view_directory(c, "view", stra(r_path));

   /* Footer */
   targeted_footer(c, stra(r_path), false);
   targeted_end(c);
   strreleasea(&r_path);
   
}


void targeted_view_qstr(const config *c, const char *in_r_path) {
/* Views a directory listing (old interface) */

   assert(c != NULL);
   assert(in_r_path != NULL);
   
   _targeted_view(c, in_r_path);
   
}


void targeted_view(const config *c) {
/* Views a directory listing */

   const char *path;

   assert(c != NULL);
   
   path = lookup_cgi_argument(c->env.query_args, "path", "");
   _targeted_view(c, path);
   
}
