/* $Header: /fridge/cvs/cgi.targeted/targeted-viewimage.c,v 1.12 2002/07/29 00:35:40 justins Exp $ */
/*
   
   targeted-viewimage.c       Copyright(c) 2002-1998 Justin David Smith
   justins(at)chaos2.org      http://chaos2.org/
    
   Functions for displaying an image
    

   This program is free software; you can redistribute it and/or modify 
   it under the terms of the GNU General Public License as published by 
   the Free Software Foundation, version 2 of the License ONLY.

   This program is distributed in the hope that it will be useful, 
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation, 
   Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/
#include <targeted-util.h>
#include <targeted-thumb.h>
#include <targeted-config.h>
#include <targeted-viewimage.h>
#include <targeted-list.h>
#include <targeted-view.h>
#include <targeted-image.h>
#include <targeted-nav.h>
#include <targeted-log.h>
#include <targeted-cgi.h>


static int _rescale_sizes[] = { 50, 66, 75, 100 };
static int _clip_widths[] = { 320, 640, 800, 1024 };


static void _viewimage_url(const config *c, const char *path, int rescale, int clipwidth) {
/* Print a viewimage URL */

   char buf[0x1000];
   bintree *cgi;

   /* Rewrite the rescale and clipwidth parametres */
   cgi = clone_cgi_arguments(c->env.query_args);
   set_cgi_argument(cgi, "mode", "viewimage");
   set_cgi_argument(cgi, "path", path);
   set_cgi_argument_int(cgi, "rescale", rescale == 100 ? 0 : rescale);
   set_cgi_argument_int(cgi, "clipwidth", clipwidth == 1 << 30 ? 0 : clipwidth);
   construct_cgi_url(c, cgi, NORMAL_URL, buf, sizeof(buf));
   printf("\"%s\"", buf);

   bintree_release(&cgi);

}
 

static void _rescale_bar(const config *c, const char *r_path) {
/* Display the resize options */

   int i;

   printf("<p>Rescale to");
   for(i = 0; i < sizeof(_rescale_sizes) / sizeof(int); ++i) {
      if(i > 0) printf(" |");
      printf(" <a title=\"Rescale image to %d%%\" href=", _rescale_sizes[i]);
      _viewimage_url(c, r_path, _rescale_sizes[i], 1 << 30);
      printf(">%d%%</a>", _rescale_sizes[i]);
   }
   printf(" of original size<br>\n");

   printf("Clip width to");
   for(i = 0; i < sizeof(_clip_widths) / sizeof(int); ++i) {
      if(i > 0) printf(" |");
      printf(" <a title=\"Clip image width to %d pixels\" href=", _clip_widths[i]);
      _viewimage_url(c, r_path, 100, _clip_widths[i]);
      printf(">%d</a>", _clip_widths[i]);
   }
   printf(" pixels wide</p>\n");

}


static void _targeted_view_image(const config *c, const char *r_path, int rescale, int clipwidth) {
/* View an image at full size; this is the internal function 
   that takes all options necessary to generate the image. */

   image_desc *desc;
   char path[0x1000];
   char parent[0x1000];
   const char *d_name;
   const char *prev;
   const char *next;
   bintree *imglist;
   bintree *imagecgi;
   bintree *cgi;
   double scalar;
   int scalewidth;
   int scaleheight;
   bool use_cgi_to_scale;
   
   /* Validate the input values */
   if(rescale <= 0) rescale = 100;
   if(clipwidth <= 0) clipwidth = 1 << 30;

   /* Currently disallow expanding */
   if(rescale > 100) rescale = 100;

   /* Header information */
   targeted_begin(c, r_path);
   printf("<h1>Image: %s</h1>\n", r_path);
   
   /* Validate the image name */
   if(!targeted_view_okay(c, r_path, false)) {
      if(targeted_view_okay(c, r_path, true)) {
         cgi = clone_cgi_arguments(c->env.query_args);
         set_cgi_argument(cgi, "mode", "thumb");
         set_cgi_argument(cgi, "path", r_path);
         construct_cgi_url(c, cgi, NORMAL_URL, path, sizeof(path));
         printf("<p class=error>You specified the IMAGE protocol, but you gave me a directory name!\n"
                "This probably happened because you thought you could chop off the file component of\n"
                "an IMAGE URI.  The script does not generate these URI's itself, so it's quite likely\n"
                "that you attempted to modify the URI yourself; please note that the script is not\n"
                "very accomodating of manual URI entry.  Maybe you want the THUMB protocol?\n"
                "Try <a href=\"%s\">this link</a> instead.</p>\n", path);
         printf("<hr>\n");
         bintree_release(&cgi);
      } else {
         targeted_warning();
      }
      targeted_end(c);
      return;
   }
   
   /* Clone the CGI arguments */
   cgi = clone_cgi_arguments(c->env.query_args);
   
   /* Get the image size */
   desc = new_image_desc(c, r_path);
   if(desc == NULL) {
      printf("<p class=error>Invalid image specified.</p>\n");
      targeted_warning();
   } else {
      printf("<div class=center>\n");

      /* Get the image name/d_name */
      d_name = r_path + strlenn(r_path);
      while(d_name > r_path && *d_name != '/') --d_name;
      if(*d_name == '/') ++d_name;

      /* Try to find out the parent */
      if(compute_parent_directory(parent, sizeof(parent), r_path)) {
         /* Try to find prev/next files */
         imglist = targeted_list_dir(c, parent, is_image_file);
         if(imglist != NULL) {
            /* Try to get prev/next images */
            prev = targeted_prev(c, imglist, d_name);
            next = targeted_next(c, imglist, d_name);
            
            /* Print out the guidelines */
            printf("<p>");
            if(prev != NULL) {
               qualify_rel_path_file(c, path, sizeof(path), parent, prev);
               printf("<a title=\"View previous image: %s\" href=", prev);
               _viewimage_url(c, path, rescale, clipwidth);
               printf(">&lt;- Prev: %s</a> | ", prev);
            } else {
               printf("Prev: None | ");
            }
            
            set_cgi_argument(cgi, "mode", "thumb");
            set_cgi_argument(cgi, "path", parent);
            construct_cgi_url(c, cgi, NORMAL_URL, path, sizeof(path));
            printf("<a title=\"View thumbnails for current directory\" href=\"%s\">Up: Thumbnail List</a>", path);
            if(next != NULL) {
               qualify_rel_path_file(c, path, sizeof(path), parent, next);
               printf(" | <a title=\"View next image: %s\" href=", next);
               _viewimage_url(c, path, rescale, clipwidth);
               printf(">Next: %s -&gt;</a>", next);
            } else {
               printf(" | Next: None");
            }
            printf("</p>");
            
            /* Release the directory list */
            bintree_release(&imglist);
         } /* Trying to output guides */
      } /* Try to get parent name */
      _rescale_bar(c, r_path);
      
      /* Print the image itself */
      scalar = rescale / 100.0;
      scalewidth  = scalar * desc->width;
      if(scalewidth > clipwidth) scalar *= (double)clipwidth / scalewidth;
      scalewidth  = scalar * desc->width;
      scaleheight = scalar * desc->height;
      use_cgi_to_scale = !(scalewidth == desc->width && scaleheight == desc->height);
      
      printf("<p>");
      if(use_cgi_to_scale) printf("Using CGI to rescale, this may take a moment...<br>\n");
      printf("<a title=\"Download the image file: %s\" href=\"%s\"><img class=border src=\"", r_path, r_path);
      if(!use_cgi_to_scale) {
         /* Unscaled image is passed through verbatim */
         printf("%s%s", stra(c->dir_prefix_uri), r_path);
      } else {
         /* Scaled images go through the targeted script */
         imagecgi = clone_cgi_arguments(c->env.query_args);
         set_cgi_argument(cgi, "mode", "image");
         set_cgi_argument(cgi, "path", r_path);
         set_cgi_argument_int(cgi, "width", scalewidth);
         set_cgi_argument_int(cgi, "height", scaleheight);
         construct_cgi_url(c, cgi, IMAGE_URL, path, sizeof(path));
         printf("%s", path);
         bintree_release(&imagecgi);
      }
      printf("\" width=%d height=%d alt=\"Image %s\"></a><br>\n", scalewidth, scaleheight, r_path);
      printf("%s (%dx%d, ", d_name, desc->width, desc->height);
      print_file_size(stdout, desc->size);
      printf(")\n");
      if(scalewidth != desc->width) {
         printf("<br>(rescaled to %dx%d)\n", scalewidth, scaleheight);
      }
      printf("</p>\n");
      _rescale_bar(c, r_path);
      if(rescale > 100) {
         printf("<p>Note: image was scaled above 100%%. The\n");
         printf("image may be somewhat blurry as a result.</p>\n");
      } else if(clipwidth > desc->width && clipwidth != 1 << 30) {
         printf("<p>Note: image was not rescaled; the clip width only\n");
         printf("determines the maximum width of the image.</p>\n");
      }
      printf("</div>\n");
      printf("<hr>\n");
      free_image_desc(&desc);
   }
   
   /* Footer information */
   targeted_end(c);
   
   /* Release the CGI args */
   bintree_release(&cgi);

}


void targeted_view_image_qstr(const config *c, const char *_r_path) {
/* View an image at full size (old interface) */

   stringa *r_path;
   stringa *arg;
   int rescale;
   int clipwidth;

   assert(c != NULL);
   assert(_r_path != NULL);
   
   /* Check to see if _r_path contains an image scale or not */
   rescale = 100;
   clipwidth = 1 << 30;
   r_path = strnewa(NULL);
   if(getnumargs(_r_path, ':') == 2) {
      arg = strnewa(NULL);
      getarga(arg, _r_path, 0, ':');
      if(*stra(arg) == 'P') {
         rescale = atoi(stra(arg) + 1);
         if(rescale <= 0) rescale = 100;
      } else if(*stra(arg) == 'W') {
         clipwidth = atoi(stra(arg) + 1);
         if(clipwidth <= 0) clipwidth = 1 << 30;
      }
      getarga(r_path, _r_path, 1, ':');
      strreleasea(&arg);
   } else {
      strcopya(r_path, _r_path);
   }

   /* Call the internal function to generate the image page */
   _targeted_view_image(c, stra(r_path), rescale, clipwidth);
   strreleasea(&r_path);
   
}


void targeted_view_image(const config *c) {
/* View an image at full size */

   const char *path;
   int rescale;
   int clipwidth;

   assert(c != NULL);
   
   /* Lookup parametres for the image viewer */
   path = lookup_cgi_argument(c->env.query_args, "path", "");
   rescale = atoi(lookup_cgi_argument(c->env.query_args, "rescale", "0"));
   clipwidth = atoi(lookup_cgi_argument(c->env.query_args, "clipwidth", "0"));
   
   /* Call the internal function to generate the image page */
   _targeted_view_image(c, path, rescale, clipwidth);

}
