/* $Header: /fridge/cvs/cgi.targeted/targeted.c,v 1.20 2002/07/03 02:19:24 justins Exp $ */
/*
   
   targeted.c                 Copyright(c) 2002-1998 Justin David Smith
   justins(at)chaos2.org      http://chaos2.org/
    
   Main program
    

   This program is free software; you can redistribute it and/or modify 
   it under the terms of the GNU General Public License as published by 
   the Free Software Foundation, version 2 of the License ONLY.

   This program is distributed in the hope that it will be useful, 
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation, 
   Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/
#include <targeted.h>
#include <targeted-util.h>
#include <targeted-config.h>
#include <targeted-tree.h>
#include <targeted-thumb.h>
#include <targeted-image.h>
#include <targeted-view.h>
#include <targeted-viewimage.h>
#include <targeted-nav.h>
#include <targeted-log.h>
#include <targeted-cgi.h>
#include <sys/stat.h>
#include <unistd.h>


void targeted_begin(const config *c, const char *title) {

   assert(c != NULL);
   assert(title != NULL);

   cgi_html_head_begin(&c->cgi, title, COPYRIGHT_NOTICE, is_typical_cgi_url(c->env.query_args));
   printf("<!-- NS 4.x cannot handle the toolbox, so hide the stylesheet from them -->\n");
   printf("<link rel=\"stylesheet\" href=\"%s\" type=\"text/css\" media=all>\n", stra(c->css_uri));
   
   /* End of header */
   printf("</head>\n");
   printf("<body>\n");
   
   if(!is_legal_cgi_url(c->env.query_args)) {
      printf("<p class=error>Note: The URL you entered contained bogus values."
             " The bogus values will be automatically ignored.</p>\n");
   }

}


void targeted_end(const config *c) {

   printf("<div class=%s>\n", c->use_toolbox ? "copyright" : "center");
   printf("<table class=footer summary=\"Footer for page\"><tr>\n");
   printf("<td>\n");
   printf("<p><a title=\"About " PACKAGE " ...\" href=\"%s?mode=about\">" PACKAGE " " VERSION "</a> by\n", stra(c->cgi_uri));
   printf("<a title=\"Chaos2 (Author's homepage)\" href=\"http://chaos2.org./\">chaos2</a><br>" COPYRIGHT_NOTICE "<br>\n");
   printf("Time elapsed:  %1.2f seconds</p>\n", time_elapsed());
   printf("<p><span class=info>");
   cgi_html_validator_links(&c->env);
   printf("</span></p>\n");
   printf("</td><td>\n");
   printf("<a title=\"Return to top of directory tree\" class=noline href=\"%s/\"><img class=noborder src=\"%s\" alt=\"Return to top of directory tree\" width=64 height=48></a>\n", stra(c->dir_prefix_uri), stra(c->icon_uri));
   printf("</td>\n");
   printf("</tr></table>\n");
   printf("</div>\n");
   printf("</body></html>\n");
   
}


void targeted_about(const config *c) {

   targeted_begin(c, "About");
   printf("<h1>About the Targeted Server</h1>\n");
   printf("<p class=justify>The Targeted server was written by Justin David Smith as a system to dynamically "
          "control content served up on the underlying targeted storage server to the world "
          "wide web.  You may also be interested in the <a title=\"Targeted Development Pages\" href=\"http://chaos2.org./software/cgi.targeted/\">Targeted Development Page</a>.</p>");
   cgi_html_gpl();
   targeted_end(c);

}


void targeted_warning(void) {

   printf("<p class=error style=\"font-size: 1.2em;\">"
          "If you attempted to enter a URL directly, then be advised:  the system is <b>very</b> "
          "finnicky, and you are <b>strongly discouraged</b> from entering URLs directly to the <i>targeted</i> "
          "system.  A large number of aliases and restrictions are in place which make manual navigation of the "
          "tree fairly difficult!  Please use the normal links which are on each page to view directories and files "
          "instead of attempting to enter URLs directly.</p>\n");

}


int main(int argc, char **argv) {

   config *c = NULL;
   const char *p;
   
   timer_init();

   /* Initialise image system */
   init_image_system();
   
   if((c = get_config(TARGETED_CONFIG_FILE))) {
      /* Perform an action. Note, "normal" actions will not display a copyright
         notice at the end of the output and will not log the action to targeted
         (since normal actions are usually "You are visitor #blah" anyway) */
      char *qstr = stra(c->env.query_string);
      const bintreenode *node;
   
      /* Log the user's connection */
      targeted_log_request(c);
      if(c->env.query_edit) targeted_log_message(c, "browser sent malformed query_string");
      
      /* Check for a new-age mode argument */
      node = bintree_pair_find_first(c->env.query_args, "mode");
      if(node != NULL) {
         /* CGI arguments were specified using the mode notation */
         const char *mode = ((bintreepair *)node->data)->value;
         if(strequal(mode, "image")) {
            /* Special image mode; do not emit normal HTML mimetype */
            targeted_image(c);
         } else {
            /* All of these are normal HTML modes */
            cgi_html_init(PACKAGE, VERSION, COPYRIGHT_NOTICE);
            if(strequal(mode, "viewimage")) {
               targeted_view_image(c);
            } else if(strequal(mode, "viewlog")) {
               targeted_view_log(c);
            } else if(strequal(mode, "viewerror")) {
               targeted_view_error(c);
            } else if(strequal(mode, "thumb")) {
               targeted_thumb(c);
            } else if(strequal(mode, "view")) {
               targeted_view(c);
            } else if(strequal(mode, "tree")) {
               targeted_tree(c);
            } else if(strequal(mode, "about")) {
               targeted_about(c);
            } else {
               targeted_log_error(c, "bogus or no action specified");
               targeted_begin(c, "No Action!");
               printf("<h1>Targeted Server: No Action!</h1>\n");
               printf("<p class=error><b>targeted error:</b> No action specified.</p>\n");
               targeted_end(c);
            } /* Which mode was specified? */
         } /* (Used for non-HTML modes) */
      } else {
         /* Using old-form notation */
         cgi_html_init(PACKAGE, VERSION, COPYRIGHT_NOTICE);
         if(     (p = (rkstrpat(NULL, qstr, "view=")))) targeted_view_qstr(c, p+5);
         else if((p = (rkstrpat(NULL, qstr, "tree=")))) targeted_tree_qstr(c, p+5);
         else if((p = (rkstrpat(NULL, qstr, "thumb=")))) targeted_thumb_qstr(c, p+6);
         else if((p = (rkstrpat(NULL, qstr, "viewimage=")))) targeted_view_image_qstr(c, p+10);
         else if((p = (rkstrpat(NULL, qstr, "viewlog=")))) targeted_view_log_qstr(c, p+8);
         else if((p = (rkstrpat(NULL, qstr, "about=")))) targeted_about(c);
         else {
            targeted_log_error(c, "bogus or no action specified");
            targeted_begin(c, "No Action!");
            printf("<h1>Targeted Server: No Action!</h1>\n");
            printf("<p class=error><b>targeted error:</b> No action specified.</p>\n");
            targeted_end(c);
         } /* Which (old) mode specified? */
      } /* Which notation is being used here? */
   } else {
      printf("<p><b>targeted error:</b> Failed to get configuration.</p>\n");
      return(1);
   }
      
   free_config(c);
   return(0);   

}
